#include "pacula.h"

Creature::Creature(	Map*		map,
					Point*		startCell,
					BITMAP**	images) :
	map(map),
	images(images),
	direction(DIR_NONE),
	directionNext(DIR_NONE)
{
	setPositionCell(startCell);
	currentImage = images[0];
	
	Point positionDest;
	getPositionPixel(&positionDest);
	Point offset;
	map->getOffset(&offset);
	lastDrawPosition = Point(positionDest.x + offset.x, positionDest.y + offset.y);
	tunnelSequence = 0;
	spriteImage = create_bitmap(map->getCellWidth(), map->getCellWidth());
}

Creature::~Creature()
{
	if (spriteImage)
	{
		destroy_bitmap(spriteImage);
	}
}


bool Creature::inHorizontalTransit()
{
	return ((position.x - map->getWallWidth()) %
		   (map->getWallWidth() + map->getCellWidth())) != 0;
}


bool Creature::inVerticalTransit()
{
	return ((position.y - map->getWallWidth()) %
		   (map->getWallWidth() + map->getCellWidth())) != 0;
}


void Creature::up()
{
	directionNext = DIR_UP;
}


void Creature::down()
{
	directionNext = DIR_DOWN;
}

void Creature::left()
{
	directionNext = DIR_LEFT;
}

void Creature::right()
{
	directionNext = DIR_RIGHT;
}

void Creature::setPositionCell(Point* cell)
{
	position.x = cell->x *
					  (map->getCellWidth() + map->getWallWidth())
					  + map->getWallWidth();
	position.y = cell->y *
					  (map->getCellWidth() + map->getWallWidth())
					  + map->getWallWidth();
}

void Creature::processSprite()
{
	blit(currentImage, spriteImage, 0, 0, 0, 0, spriteImage->w, spriteImage->h);
}

void Creature::draw(BITMAP* bitmap)
{
	Point positionDest;
	getPositionPixel(&positionDest);
	Point offset;
	map->getOffset(&offset);
	
	processSprite();
	
	if (tunnelSequence)
	{
		int rotateFactor = tunnelSequence * 5;
		double scaleFactor = fabs((double)(tunnelSequence-128)/128.0);
		int pivotOffset = map->getCellWidth()/2;
		
		set_clip_rect(	bitmap,
						positionDest.x + offset.x,
						positionDest.y + offset.y,
						positionDest.x + offset.x + map->getCellWidth()-1,
						positionDest.y + offset.y + map->getCellWidth()-1);
		
		pivot_scaled_sprite(	bitmap,
								spriteImage,
								positionDest.x + offset.x + pivotOffset,
								positionDest.y + offset.y + pivotOffset,
								pivotOffset,
								pivotOffset,
								itofix(rotateFactor),
								ftofix(scaleFactor));
		
		set_clip_rect(bitmap, 0, 0, bitmap->w, bitmap->h);

		// Move to the other tunnel halfway through the sequence
		if (tunnelSequence == 128)
		{
			Point cellPosition;
			getPositionCell(&cellPosition);
			if (cellPosition.x == 0)
			{
				cellPosition.x = MAP_WIDTH - 1;
			}
			else
			{
				cellPosition.x = 0;
			}
			setPositionCell(&cellPosition);
		}
		
		--tunnelSequence;
	}
	else
	{
		draw_sprite(bitmap, spriteImage, positionDest.x + offset.x, positionDest.y + offset.y);
	}
	
	lastDrawPosition = Point(positionDest.x + offset.x, positionDest.y + offset.y);
}

void Creature::cleanup(BITMAP* bitmap)
{
	rectfill(	bitmap,
				lastDrawPosition.x,
				lastDrawPosition.y,
				lastDrawPosition.x + map->getCellWidth() - 1,
				lastDrawPosition.y + map->getCellWidth() - 1,
				0);
}

void Creature::decrementX(Cell* cell)
{
	if ((position.x <= map->getWallWidth()) && cell->canMoveLeft())
	{
		tunnelSequence = 256;
		return;
	}
					
	position.x--;
}

void Creature::incrementX(Cell* cell, Point* cellPosition)
{
	if ((cellPosition->x >= MAP_WIDTH-1) && cell->canMoveRight())
	{
		tunnelSequence = 256;
		return;
	}
					
	position.x++;
}

void Creature::decrementY(Cell* cell)
{
	if ((position.y <= map->getWallWidth()) && cell->canMoveUp())
	{
		tunnelSequence = 256;
		return;
	}
					
	position.y--;
}

void Creature::incrementY(Cell* cell, Point* cellPosition)
{
	if ((cellPosition->y >= MAP_HEIGHT-1) && cell->canMoveDown())
	{
		tunnelSequence = 256;
		return;
	}
					
	position.y++;
}

void Creature::doFrame()
{
	if (tunnelSequence)
	{
		return;
	}
	
	Point cellPosition;
	getPositionCell(&cellPosition);
	Cell* cell = map->getCell(&cellPosition);
	
	if (inVerticalTransit())
	{
		if (direction == DIR_UP)
		{
			if (directionNext == DIR_DOWN)
			{
				direction = DIR_DOWN;
				incrementY(cell, &cellPosition);
			}
			else
			{
				decrementY(cell);
			}
		}
		else
		{
			if (directionNext == DIR_UP)
			{
				direction = DIR_UP;
				decrementY(cell);
			}
			else
			{
				incrementY(cell, &cellPosition);
			}
		}
	}
	else if (inHorizontalTransit())
	{
		if (direction == DIR_LEFT)
		{
			if (directionNext == DIR_RIGHT)
			{
				direction = DIR_RIGHT;
				incrementX(cell, &cellPosition);
			}
			else
			{
				decrementX(cell);
			}
		}
		else
		{
			if (directionNext == DIR_LEFT)
			{
				direction = DIR_LEFT;
				decrementX(cell);
			}
			else
			{
				incrementX(cell, &cellPosition);
			}
		}
	}
	else
	{
		switch (directionNext)
		{
			case DIR_UP:
				if (cell->canMoveUp())
				{
					direction = DIR_UP;
					decrementY(cell);
					return;
				}
				break;
			
			case DIR_DOWN:
				if (cell->canMoveDown())
				{
					direction = DIR_DOWN;
					incrementY(cell, &cellPosition);
					return;
				}
				break;
			
			case DIR_LEFT:
				if (cell->canMoveLeft())
				{
					direction = DIR_LEFT;
					decrementX(cell);
					return;
				}
				break;
			
			case DIR_RIGHT:
				if (cell->canMoveRight())
				{
					direction = DIR_RIGHT;
					incrementX(cell, &cellPosition);
					return;
				}
		}
		
//		directionNext = direction;
		
		switch (direction)
		{
			case DIR_UP:
				if (cell->canMoveUp())
				{
					decrementY(cell);
				}
				else
				{
					direction = DIR_NONE;
				}
				break;
				
			case DIR_DOWN:
				if (cell->canMoveDown())
				{
					incrementY(cell, &cellPosition);
				}
				else
				{
					direction = DIR_NONE;
				}
				break;
					
			case DIR_LEFT:
				if (cell->canMoveLeft())
				{
					decrementX(cell);
				}
				else
				{
					direction = DIR_NONE;
				}
				break;
					
			case DIR_RIGHT:
				if (cell->canMoveRight())
				{
					incrementX(cell, &cellPosition);
				}
				else
				{
					direction = DIR_NONE;
				}
		}
		
		hitWall = (direction == DIR_NONE);
	}
}


void Creature::getPositionCell(Point* cell)
{
	cell->x = (position.x - map->getWallWidth()) / (map->getWallWidth() + map->getCellWidth());
	cell->y = (position.y - map->getWallWidth()) / (map->getWallWidth() + map->getCellWidth());
}


void Creature::getPositionPixel(Point* position)
{
	position->x = this->position.x;
	position->y = this->position.y;
}


void Creature::getAbsolutePosition(Point* position)
{
	Point offset;
	getPositionPixel(position);
	map->getOffset(&offset);
	position->x += offset.x;
	position->y += offset.y;
}

bool Creature::collide(Creature* creature)
{
	int width = map->getCellWidth();
	if (creature->position.x+width < position.x)	return false;
	if (creature->position.x > position.x+width)	return false;
	if (creature->position.y+width < position.y)	return false;
	if (creature->position.y > position.y+width)	return false;
	return true;
}


bool Creature::hasHitWall()
{
	return hitWall;
}


int Creature::getDirection()
{
	return direction;
}
