#include "pacula.h"

void buildPaculaImages(BITMAP** images, DATAFILE* data);
void buildGhostImages(BITMAP** images, DATAFILE* data);
void buildTrollImages(BITMAP**images, DATAFILE* data);
void buildPelletImages(BITMAP** images, DATAFILE* data);
		
enum
{
	PLAYER_PREDATOR,
	PLAYER_PREY
};

class Player : public Creature
{
protected:
	int faceDirection;
	int state;

public:
	Player(	Map*		map,
			Point*		startCell,
			BITMAP**	images) : Creature(map, startCell, images)
	{
		faceDirection = DIR_RIGHT;
		state = PLAYER_PREY;
	}
	
	~Player()
	{
	}
	
	void doFrame()
	{
		// Move
		Creature::doFrame();

		// Decide which image is the current one
		int imageIndex = 0;
		int cellWidth = map->getCellWidth() + map->getWallWidth();
		int offset = abs(position.x % cellWidth) + abs(position.y % cellWidth);
		if (offset < cellWidth / 2)
		{
			imageIndex = 1;
		}
		else if (offset <= (3*cellWidth) / 4)
		{
			imageIndex = 2;
		}
		
		if (state == PLAYER_PREDATOR)
		{
			imageIndex += 3;
		}
		currentImage = images[imageIndex];
		
		// Decide which way the image will face
		if (direction != DIR_NONE)
		{
			faceDirection = direction;
		}
	}
	
	void setState(int state)
	{
		this->state = state;
	}
	
	void processSprite()
	{
		clear_bitmap(spriteImage);
		switch (faceDirection)
		{
			case DIR_UP:
				rotate_sprite_v_flip(spriteImage, currentImage, 0, 0, itofix(192));
				break;
			case DIR_DOWN:
				rotate_sprite(spriteImage, currentImage, 0, 0, itofix(64));
				break;
			case DIR_LEFT:
				draw_sprite_h_flip(spriteImage, currentImage, 0, 0);
				break;
			case DIR_RIGHT:
				draw_sprite(spriteImage, currentImage, 0, 0);
		}
	}
};

enum
{
	GHOST_HOME,
	GHOST_PREDATOR,
	GHOST_PREY,
	GHOST_HOMING
};
	
#define PREDATOR_OFFSET_0	0
#define PREDATOR_OFFSET_1	2
#define PREDATOR_OFFSET_2	4
#define PREDATOR_OFFSET_3	6
#define PREY_OFFSET			8
#define EYES_OFFSET			10

class Ghost : public Creature
{
protected:
	int state;
	int homeTimer;
	int homeTimerStart;
	int preyTimer;
	int imageBaseIndex;
	Creature* player;
	bool goHome;

public:
	Ghost(	Map*		map,
			Point*		startCell,
			BITMAP**	images,
			int			imageBaseIndex,
			int			homeTimerStart,
			Creature*	player) : Creature(map, startCell, images)
	{
		this->imageBaseIndex = imageBaseIndex;
		this->homeTimerStart = homeTimerStart;
		homeTimer = homeTimerStart;
		preyTimer = 0;
		state = GHOST_HOME;
		this->player = player;
		goHome = false;
	}
	
	virtual ~Ghost()
	{
	}
	
	void sendHome()
	{
		state = GHOST_HOMING;
		preyTimer = 0;
	}
	
	int getState()
	{
		return state;
	}
	
	void scare()
	{
		state = GHOST_PREY;
	}
	
	void unscare()
	{
		if (state = GHOST_PREY)
		{
			state = GHOST_PREDATOR;
		}
	}
	
	virtual void pickDirection(Cell* cell) = 0;
	
	
	void doFrame()
	{
		if (tunnelSequence)
		{
			return;
		}
		
		int cellWidth = map->getCellWidth() + map->getWallWidth();
		bool inTransit = inVerticalTransit() | inHorizontalTransit();
		Point ghostHome;
		map->getGhostHome(&ghostHome);
		
		Point cellPosition;
		getPositionCell(&cellPosition);
		Cell* cell = map->getCell(&cellPosition);
		
		switch (state)
		{
			case GHOST_HOME:
				// Check for state change
				if (--homeTimer <= 0)
				{
					state = GHOST_PREDATOR;
					up();
				}
				break;
			case GHOST_PREDATOR:
				if (!inTransit)
				{
					pickDirection(cell);
				}
				break;
			case GHOST_PREY:
				if (!inTransit)
				{
					pickDirection(cell);
					
					// Check for state change
					if (goHome)
					{
						state = GHOST_HOMING;
						goHome = false;
					}
				}
			case GHOST_HOMING:
				if (!inTransit)
				{
					// Check for state change
					if (cellPosition == ghostHome)
					{
						state = GHOST_HOME;
						homeTimer = homeTimerStart;
					}
					// Otherwise find our way home
					else
					{
						if ((cellPosition.y > 0) &&
							cell->canMoveUp() &&
							(map->homingMap[cellPosition.x][cellPosition.y-1] > map->homingMap[cellPosition.x][cellPosition.y]))
						{
							up();
						}
						else if (	(cellPosition.y < MAP_HEIGHT-1) &&
									cell->canMoveDown() &&
									(map->homingMap[cellPosition.x][cellPosition.y+1] > map->homingMap[cellPosition.x][cellPosition.y]))
						{
							down();
						}
						else if (	(cellPosition.x > 0) &&
									cell->canMoveLeft() &&
									(map->homingMap[cellPosition.x-1][cellPosition.y] > map->homingMap[cellPosition.x][cellPosition.y]))
						{
							left();
						}
						else if (	(cellPosition.x < MAP_WIDTH-1) &&
									cell->canMoveRight() &&
									(map->homingMap[cellPosition.x+1][cellPosition.y] > map->homingMap[cellPosition.x][cellPosition.y]))
						{
							right();
						}
					}
				}
		}
		
		Creature::doFrame();
	}
	
	void processSprite()
	{
		clear_bitmap(spriteImage);
		int imageIndex;
		
		switch (state)
		{
			case GHOST_HOME:
			case GHOST_PREDATOR:
				switch ((position.x >> 3) + (position.y >> 3))
				{
					case 1:
					case 3:
						imageIndex = imageBaseIndex;
						break;
					default:
						imageIndex = imageBaseIndex+1;
				}
				draw_sprite(spriteImage, images[imageIndex], 0, 0);
				
				// Add the eyes
				imageIndex = EYES_OFFSET;
				if (state != GHOST_HOME)
				{
					switch (direction)
					{
						case DIR_DOWN:
							imageIndex += 1;
							break;
						case DIR_LEFT:
							imageIndex += 2;
							break;
						case DIR_RIGHT:
							imageIndex += 3;
					}
				}
				draw_sprite(spriteImage, images[imageIndex], 0, 0);
				break;
						
			case GHOST_PREY:
				switch ((position.x >> 3) + (position.y >> 3))
				{
					case 1:
					case 3:
						imageIndex = PREY_OFFSET;
						break;
					default:
						imageIndex = PREY_OFFSET+1;
				}
				draw_sprite(spriteImage, images[imageIndex], 0, 0);
				break;
			case GHOST_HOMING:
				imageIndex = EYES_OFFSET;
				switch (direction)
				{
					case DIR_DOWN:
						imageIndex += 1;
						break;
					case DIR_LEFT:
						imageIndex += 2;
						break;
					case DIR_RIGHT:
						imageIndex += 3;
				}
				draw_sprite(spriteImage, images[imageIndex], 0, 0);
		}
	}
	
};

class GhostType1 : public Ghost
{
protected:
	int aggression, fear, memory; // 0 = never. 10 = always

public:
	GhostType1(	Map*		map,
				Point*		startCell,
				BITMAP**	images,
				int			imageBaseIndex,
				int			homeTimerStart,
				Player*		player,
				int			aggression,
				int			fear,
				int			memory) : Ghost(map, startCell, images, imageBaseIndex, homeTimerStart, player)
	{
		this->aggression = aggression;
		this->fear = fear;
		this->memory = memory;
	}
	
	~GhostType1()
	{
	}
	
	void pickDirection(Cell* cell)
	{
		switch (state)
		{
			case GHOST_PREDATOR:
				if ((rand()%10) < aggression)
				{
					Point playerPos;
					player->getPositionPixel(&playerPos);
					
					if ((playerPos.y < position.y) && cell->canMoveUp())
					{
						up();
						
					}
					else if ((playerPos.y > position.y) && cell->canMoveDown())
					{
						down();
					}
					else if ((playerPos.x < position.x) && cell->canMoveLeft())
					{
						left();
					}
					else if ((playerPos.x > position.x) && cell->canMoveRight())
					{
						right();
					}
					else // Can't follow player, so pick a random direction
					{
						switch (pickRandomDirection(cell, direction, memory))
						{
							case DIR_UP:
								up();
								break;
							case DIR_DOWN:
								down();
								break;
							case DIR_LEFT:
								left();
								break;
							case DIR_RIGHT:
								right();
						}
					}
				}
				else // Non-aggressive move; just random
				{
					switch (pickRandomDirection(cell, direction, memory))
					{
						case DIR_UP:
							up();
							break;
						case DIR_DOWN:
							down();
							break;
						case DIR_LEFT:
							left();
							break;
						case DIR_RIGHT:
							right();
					}
				}
				break;
				
			case GHOST_PREY:
				if ((rand()%10) < fear)
				{
					Point playerPos;
					player->getPositionPixel(&playerPos);
					
					if (playerPos.y < position.y && cell->canMoveDown())
					{
						down();
					}
					else if (playerPos.y > position.y && cell->canMoveUp())
					{
						up();
					}
					else if (playerPos.x < position.x && cell->canMoveRight())
					{
						right();
					}
					else if (playerPos.x > position.x && cell->canMoveLeft())
					{
						left();
					}
					else // Can't flea from player, so pick a random direction
					{
						switch (pickRandomDirection(cell, direction, memory))
						{
							case DIR_UP:
								up();
								break;
							case DIR_DOWN:
								down();
								break;
							case DIR_LEFT:
								left();
								break;
							case DIR_RIGHT:
								right();
						}
					}
				}
				else // Tried to flea from player, but blocked
				{
					switch (pickRandomDirection(cell, direction, memory))
					{
						case DIR_UP:
							up();
							break;
						case DIR_DOWN:
							down();
							break;
						case DIR_LEFT:
							left();
							break;
						case DIR_RIGHT:
							right();
					}
				}
		}
	}
	
	int pickRandomDirection(Cell* cell, int direction, int memory)
	{
		// If this ghost forgets where it's going, make it backtrack
		if ((rand()%10) < memory)
		{
			switch (direction)
			{
				case DIR_UP:
					if (cell->canMoveUp())	return DIR_UP;
					else
					{
						switch (rand()%2)
						{
							case 0:
								if		(cell->canMoveLeft())	return DIR_LEFT;
								else if	(cell->canMoveRight())	return DIR_RIGHT;
								else							return DIR_DOWN;
							case 1:
								if		(cell->canMoveRight())	return DIR_RIGHT;
								else if	(cell->canMoveLeft())	return DIR_LEFT;
								else							return DIR_DOWN;
						}
					}
				case DIR_DOWN:
					if (cell->canMoveDown())	return DIR_DOWN;
					else
					{
						switch (rand()%2)
						{
							case 0:
								if		(cell->canMoveLeft())	return DIR_LEFT;
								else if	(cell->canMoveRight())	return DIR_RIGHT;
								else							return DIR_UP;
							case 1:
								if		(cell->canMoveRight())	return DIR_RIGHT;
								else if	(cell->canMoveLeft())	return DIR_LEFT;
								else							return DIR_UP;
						}
					}
				case DIR_LEFT:
					if (cell->canMoveLeft())	return DIR_LEFT;
					else
					{
						switch (rand()%2)
						{
							case 0:
								if		(cell->canMoveUp())		return DIR_UP;
								else if	(cell->canMoveDown())	return DIR_DOWN;
								else							return DIR_RIGHT;
							case 1:
								if		(cell->canMoveDown())	return DIR_DOWN;
								else if	(cell->canMoveUp())		return DIR_UP;
								else							return DIR_RIGHT;
						}
					}
				case DIR_RIGHT:
					if (cell->canMoveRight())	return DIR_RIGHT;
					else
					{
						switch (rand()%2)
						{
							case 0:
								if		(cell->canMoveUp())		return DIR_UP;
								else if	(cell->canMoveDown())	return DIR_DOWN;
								else							return DIR_LEFT;
							case 1:
								if		(cell->canMoveDown())	return DIR_DOWN;
								else if	(cell->canMoveUp())		return DIR_UP;
								else							return DIR_LEFT;
						}
					}
			}
		}
		
		switch (direction)
		{
			case DIR_UP:
				switch (rand()%2)
				{
					case 0:
						if		(cell->canMoveLeft())		return DIR_LEFT;
						else if	(cell->canMoveRight())		return DIR_RIGHT;
						else if (cell->canMoveUp())			return DIR_UP;
						else								return DIR_DOWN;
					case 1:
						if		(cell->canMoveRight())		return DIR_RIGHT;
						else if	(cell->canMoveLeft())		return DIR_LEFT;
						else if (cell->canMoveUp())			return DIR_UP;
						else								return DIR_DOWN;
				}
			case DIR_DOWN:
				switch (rand()%2)
				{
					case 0:
						if		(cell->canMoveLeft())		return DIR_LEFT;
						else if	(cell->canMoveRight())		return DIR_RIGHT;
						else if (cell->canMoveDown())		return DIR_DOWN;
						else								return DIR_UP;
					case 1:
						if		(cell->canMoveRight())		return DIR_RIGHT;
						else if	(cell->canMoveLeft())		return DIR_LEFT;
						else if (cell->canMoveDown())		return DIR_DOWN;
						else								return DIR_UP;
				}
			case DIR_LEFT:
				switch (rand()%2)
				{
					case 0:
						if		(cell->canMoveUp())			return DIR_UP;
						else if	(cell->canMoveDown())		return DIR_DOWN;
						else if (cell->canMoveRight())		return DIR_RIGHT;
						else								return DIR_LEFT;
					case 1:
						if		(cell->canMoveDown())		return DIR_DOWN;
						else if	(cell->canMoveUp())			return DIR_UP;
						else if (cell->canMoveRight())		return DIR_RIGHT;
						else								return DIR_LEFT;
				}
			case DIR_RIGHT:
				switch (rand()%2)
				{
					case 0:
						if		(cell->canMoveUp())			return DIR_UP;
						else if	(cell->canMoveDown())		return DIR_DOWN;
						else if (cell->canMoveLeft())		return DIR_LEFT;
						else								return DIR_RIGHT;
					case 1:
						if		(cell->canMoveDown())		return DIR_DOWN;
						else if	(cell->canMoveUp())			return DIR_UP;
						else if (cell->canMoveLeft())		return DIR_LEFT;
						else								return DIR_RIGHT;
				}
		}
	}
};

struct GhostParameters
{
	BITMAP** images;
	int imageBaseIndex;
	int homeTimer;
	int speed;
	int aggression;
	int fear;
	int memory;
	
	GhostParameters(BITMAP** images,
					int imageBaseIndex,
					int homeTimer,
					int speed,
					int aggression,
					int fear,
					int memory) :
	images(images),
	imageBaseIndex(imageBaseIndex),
	homeTimer(homeTimer),
	speed(speed),
	aggression(aggression),
	fear(fear),
	memory(memory)
	{
	}
};

void doLevel(	int mapIndex,
				int* score,
				BITMAP* background1,
				BITMAP* background2,
				BITMAP** pacImages,
				BITMAP** pelletImages,
				BITMAP** trollImages,
				int playerSpeed,
				GhostParameters* ghostParams);

int main()
{
	allegro_init();
	install_keyboard();
	set_color_depth(8);
	set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0);

	DATAFILE* data;
	data = load_datafile("pacula.dat");
	set_palette((RGB*)data[GAME_PALETTE].dat);
	
	BITMAP* background1 = create_bitmap(640, 480);
	BITMAP* background2 = create_bitmap(640, 480);
	clear_bitmap(background1);
	clear_bitmap(background2);
	
	BITMAP* pacImages[6];
	buildPaculaImages(pacImages, data);
	
	BITMAP* pelletImages[5];
	buildPelletImages(pelletImages, data);
	
	BITMAP* trollImages[3];
	buildTrollImages(trollImages, data);
	
	BITMAP* ghostImages[14];
	buildGhostImages(ghostImages, data);
	
	GhostParameters ghostParams[4] =	{
											GhostParameters(ghostImages, PREDATOR_OFFSET_0, 600, 26, 1, 4, 8),
											GhostParameters(ghostImages, PREDATOR_OFFSET_1, 500, 22, 1, 5, 7),
											GhostParameters(ghostImages, PREDATOR_OFFSET_2, 400, 19, 3, 6, 6),
											GhostParameters(ghostImages, PREDATOR_OFFSET_3, 300, 17, 2, 7, 5)
										};
	
	int score = 0;
	
	doLevel(1,
			&score,
			background1,
			background2,
			pacImages,
			pelletImages,
			trollImages,
			20,
			&ghostParams[0]);
		
	return 0;
}
END_OF_MAIN()


void buildPaculaImages(BITMAP** images, DATAFILE* data)
{
	images[0] = (BITMAP*)(data[PAC_NORMAL_0].dat);
	images[1] = (BITMAP*)(data[PAC_NORMAL_1].dat);
	images[2] = (BITMAP*)(data[PAC_NORMAL_2].dat);
	images[3] = (BITMAP*)(data[PAC_EVIL_0].dat);
	images[4] = (BITMAP*)(data[PAC_EVIL_1].dat);
	images[5] = (BITMAP*)(data[PAC_EVIL_2].dat);
}

void buildGhostImages(BITMAP** images, DATAFILE* data)
{
	images[0]	= (BITMAP*)(data[GHOST_0_NORMAL_0].dat);
	images[1]	= (BITMAP*)(data[GHOST_0_NORMAL_1].dat);
	images[2]	= (BITMAP*)(data[GHOST_1_NORMAL_0].dat);
	images[3]	= (BITMAP*)(data[GHOST_1_NORMAL_1].dat);
	images[4]	= (BITMAP*)(data[GHOST_2_NORMAL_0].dat);
	images[5]	= (BITMAP*)(data[GHOST_2_NORMAL_1].dat);
	images[6]	= (BITMAP*)(data[GHOST_3_NORMAL_0].dat);
	images[7]	= (BITMAP*)(data[GHOST_3_NORMAL_1].dat);
	images[8]	= (BITMAP*)(data[GHOST_PREY_0].dat);
	images[9]	= (BITMAP*)(data[GHOST_PREY_1].dat);
	images[10]	= (BITMAP*)(data[GHOST_EYES_UP].dat);
	images[11]	= (BITMAP*)(data[GHOST_EYES_DOWN].dat);
	images[12]	= (BITMAP*)(data[GHOST_EYES_LEFT].dat);
	images[13]	= (BITMAP*)(data[GHOST_EYES_RIGHT].dat);
}

void buildTrollImages(BITMAP** images, DATAFILE* data)
{
	images[0] = (BITMAP*)(data[TROLL_0].dat);
	images[1] = (BITMAP*)(data[TROLL_1].dat);
	images[2] = (BITMAP*)(data[TROLL_2].dat);
}

void buildPelletImages(BITMAP** images, DATAFILE* data)
{
	images[0] = (BITMAP*)(data[PELLET].dat);
	images[1] = (BITMAP*)(data[POWER_PELLET_0].dat);
	images[2] = (BITMAP*)(data[POWER_PELLET_1].dat);
	images[3] = (BITMAP*)(data[POWER_PELLET_2].dat);
	images[4] = (BITMAP*)(data[POWER_PELLET_1].dat);
}

bool addScore(int* score, int a)
{
	(*score)+=a;
	printf("Score = %d\n", *score);
	if (*score >= 9999)
	{
		*score = 9999;
		return true;
	}
	
	return false;
}


void doLevel(	int mapIndex,
				int* score,
				BITMAP* background1,
				BITMAP* background2,
				BITMAP** pacImages,
				BITMAP** pelletImages,
				BITMAP** trollImages,
				int playerSpeed,
				GhostParameters* ghostParams)
{
	Point tempPoint;

	// Initialise
	int pelletTimer = 12;
	int pelletCycle = 0;
	int predatorTimer = 0;
	
	int objective = MAP_WIDTH*MAP_HEIGHT-1;
	
	// Load map
	Map map("map.txt", mapIndex, 32, 4, Point(167, 9).addr());
	map.buildHomingMap();
	clear_bitmap(background1);
	map.draw(background1, 15);
	map.drawPellets(background1, pelletImages);
	
	// Create player
	Player player(&map, map.getPacStart(&tempPoint), pacImages);
	int playerTimer = playerSpeed;
	
	// Create ghosts
	GhostType1 ghosts[4] =	{
								GhostType1(	&map,
											map.getGhostHome(&tempPoint),
											ghostParams[0].images,
											ghostParams[0].imageBaseIndex,
											ghostParams[0].homeTimer,
											&player,
											ghostParams[0].aggression,
											ghostParams[0].fear,
											ghostParams[0].memory),
	
								GhostType1(	&map,
											map.getGhostHome(&tempPoint),
											ghostParams[1].images,
											ghostParams[1].imageBaseIndex,
											ghostParams[1].homeTimer,
											&player,
											ghostParams[1].aggression,
											ghostParams[1].fear,
											ghostParams[1].memory),
	
								GhostType1(	&map,
											map.getGhostHome(&tempPoint),
											ghostParams[2].images,
											ghostParams[2].imageBaseIndex,
											ghostParams[2].homeTimer,
											&player,
											ghostParams[2].aggression,
											ghostParams[2].fear,
											ghostParams[2].memory),

								GhostType1(	&map,
											map.getGhostHome(&tempPoint),
											ghostParams[3].images,
											ghostParams[3].imageBaseIndex,
											ghostParams[3].homeTimer,
											&player,
											ghostParams[3].aggression,
											ghostParams[3].fear,
											ghostParams[3].memory)
							};

	int ghostTimers[4];
	for (int i=0; i<4; ++i)
	{
		ghostTimers[i] = ghostParams[i].speed;
	}
	
	// Do the gameplay
	while (!key[KEY_ESC])
	{
		// There are 20 timing cycles per frame
		for (int i=0; i<20; ++i)
		{
			// Countdown player timer
			if ((--playerTimer) <= 0)
			{
				// Control and animate the player
				switch (player.getDirection())
				{
					case DIR_UP:
						if (key[KEY_DOWN]	&& !key[KEY_UP])	player.down();
						if (key[KEY_LEFT]	&& !key[KEY_RIGHT])	player.left();
						if (key[KEY_RIGHT] && !key[KEY_LEFT])	player.right();
						break;
					case DIR_DOWN:
						if (key[KEY_UP]		&& !key[KEY_DOWN])	player.up();
						if (key[KEY_LEFT]	&& !key[KEY_RIGHT])	player.left();
						if (key[KEY_RIGHT]	&& !key[KEY_LEFT])	player.right();
						break;
					case DIR_LEFT:
						if (key[KEY_RIGHT] && !key[KEY_LEFT])	player.right();
						if (key[KEY_UP]		&& !key[KEY_DOWN])	player.up();
						if (key[KEY_DOWN]	&& !key[KEY_UP])	player.down();
						break;
					case DIR_RIGHT:
						if (key[KEY_LEFT]	&& !key[KEY_RIGHT])	player.left();
						if (key[KEY_UP]		&& !key[KEY_DOWN])	player.up();
						if (key[KEY_DOWN]	&& !key[KEY_UP])	player.down();
						break;
					default:
						if (key[KEY_UP]		&& !key[KEY_DOWN])	player.up();
						if (key[KEY_DOWN]	&& !key[KEY_UP])	player.down();
						if (key[KEY_LEFT]	&& !key[KEY_RIGHT])	player.left();
						if (key[KEY_RIGHT]	&& !key[KEY_LEFT])	player.right();
				}
				player.doFrame();
				playerTimer = playerSpeed;
			}
			
			// Countdown ghost timers
			for (int j=0; j<4; ++j)
			{
				if ((--ghostTimers[j]) <= 0)
				{
					// Control and animate ghost
					ghosts[j].doFrame();
					ghostTimers[j] = ghostParams[j].speed;
				}
			}
			
			Point p;
			player.getPositionCell(&p);
			Cell* c = map.getCell(&p);
			if (!(player.inVerticalTransit() || player.inHorizontalTransit()))
			{
				if (c->hasPowerPellet())
				{
					c->eatPellet();
					if (addScore(score, 25))
					{
						return;
					}
					if ((--objective) <= 0)
					{
						return;
					}
					
					for (int j=0; j<4; ++j)
					{
						ghosts[j].scare();
					}
					predatorTimer = 1000;
					player.setState(PLAYER_PREDATOR);
				}
				else if (c->hasPellet())
				{
					c->eatPellet();
					if (addScore(score, 5))
					{
						return;
					}
					player.getAbsolutePosition(&p);
					rectfill(background1, p.x, p.y, p.x+31, p.y+31, 0);
					if ((--objective) <= 0)
					{
						return;
					}
				}
			}
			for (int j=0; j<4; ++j)
			{
				if (player.collide(dynamic_cast<Creature*>(&ghosts[j])))
				{
					switch (ghosts[j].getState())
					{
						case GHOST_PREDATOR:
							return;
						case GHOST_PREY:
							addScore(score, 500);
							ghosts[j].sendHome();
					}
				}
			}
		}
		
		if ((--pelletTimer) <= 0)
		{
			pelletTimer = 12;
			pelletCycle = (pelletCycle+1) & 3;
		}
		
		// Render everything
		blit(background1, background2, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		map.drawPowerPellets(background2, pelletImages, pelletCycle);
		player.draw(background2);
		for(int i=0; i<4; ++i)
		{
			ghosts[i].draw(background2);
		}
		vsync();
		blit(background2, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		
		// Some more stuff
		if (predatorTimer)
		{
			if ((--predatorTimer) <= 0)
			{
				for (int i=0; i<4; ++i)
				{
					ghosts[i].unscare();
				}
				player.setState(PLAYER_PREY);
			}
		}
	}
}

